# adapackagefilter.awk -- a noweb notangle filter for Ada
#   The filter does the following:
#   - it appends a "_P" suffix to names from a configuration file named
#     "packagenames.txt"
#   - it leaves all lines intact with references to "Ada", "Cairo", "Gdk",
#      "Glib", "Gtk" and "Pango"

BEGIN \
  {
      true = 1;
      false = 0;
      inCodeChunk = false;
      bufferLength = 0;

      if (configFileName == "") {
	  configFileName = "packagenames.txt";
      }

      readConfigFile();
  }

#--------------------

/.*/ \
  {
      currentLine = $0;
  }

#--------------------

/^@end code/ \
  {
      inCodeChunk = false;
      flushBuffer();
      print currentLine;
      next;
  }

#--------------------

/^@begin code/ \
  {
      inCodeChunk = true;
      flushBuffer();
      buffer[++bufferLength] = currentLine;
      next;
  }

#--------------------

/.*/ && !inCodeChunk \
  {
      print currentLine;
      next;
  }

#--------------------

/^@text/ && inCodeChunk \
  {
      effectiveText = substr(currentLine, 7, 1000);

      if (effectiveText ~ /(Ada|Cairo|Gdk|Glib|Gtk|Pango)\./ ) {
	  # leave line as is
      } else if (effectiveText ~ packageList ) {
	  for (i = 1;  i <= patternCount;  i++) {
	      gsub(packageName[i], expandedName[i], effectiveText);
	  }
      }

      currentLine = "@text " effectiveText;
      buffer[++bufferLength] = currentLine;
      next;
  }

#--------------------

/.*/ && inCodeChunk \
  {
      buffer[++bufferLength] = currentLine;
      next;
  }

#--------------------

END \
  {
      flushBuffer();
  }

#--------------------
#--------------------

function readConfigFile() \
  {
      packageList = "";
      patternCount = 0;

      while ((getline currentLine < configFileName) > 0) {
	  patternCount++;
	  gsub(/[ \r\n]+/, "", currentLine);
	  pattern = "\\y" currentLine "\\y";
	  packageName[patternCount]  = pattern;
	  expandedName[patternCount] = currentLine "_P";
	  packageList = packageList (packageList == "" ? "(" : "|") pattern;
      }
      
      packageList = packageList ")";
  }

#--------------------

function flushBuffer() \
  {
      # remove trailing blank lines
      for (i = bufferLength - 1;  i > 0;  i--) {
	  thisLine = buffer[i];
	  
	  if (thisLine ~ /^@text *$/ || thisLine ~ /^@nl$/ ) {
	      bufferLength--;
	  } else {
	      break;
	  }
      }
    
      for (i = 1;  i <= bufferLength;  i++) {
	  print buffer[i];
      }
    
      bufferLength = 0;
  }
