@ECHO OFF
REM PLOY distribution commandline script
REM builds executable ploy.exe and literate program document in
REM doc/ploy-ada.pdf

REM ===================================

SET currentDirectory=%~dp0
SET currentDirectory=%currentDirectory:~0,-1%
SET progName=%~n0

REM ===================================
REM ========== CONFIGURATION ==========
REM ===================================

CALL %currentDirectory%\_configuration.cmd

REM ====================================
REM ========== INTERNAL SETUP ==========
REM ====================================

IF NOT "%tempDirectory%" == "" GOTO ENDIF1
    SET tempDirectory=%TEMP%
:ENDIF1

SET sourceDirectory=%currentDirectory%\src
SET sourceFile=%sourceDirectory%\ploy-ada.nw
SET executableDirectory=%currentDirectory%\bin

REM --- noweb literate programming filters ---
SET notangle=CALL _support_tools\NOTANGLE-TT
SET noweave=CALL _support_tools\NOWEAVE-TT
SET adaTangle=CALL _support_tools\LOCALTANGLE-ADA
SET adaTangleP=CALL _support_tools\LOCALTANGLE-ADA-P

REM -----------------------------------------------------------------
REM -- generation of configuration file for package name expansion --
REM -----------------------------------------------------------------

SET packageNameFileWindows=%tempDirectory%\packageNames.txt
SET packageNameFile=%packageNameFileWindows:\=/%
%notangle% %sourceFile% packageNames.txt %packageNameFileWindows% text

REM ======================================================
REM ========== MAKING LITERATE PROGRAM DOCUMENT ==========
REM ======================================================

IF %processingMode% == executable GOTO ENDIF2
    SET isOkay=T
    IF "%pdfLaTeX%" == ""  SET isOkay=F

    IF NOT %isOkay% == F GOTO ELSE21
        SET message=%progName%: no PDFLaTeX processor
        SET message=%message% - cannot produce literate program document
        ECHO %message%
	GOTO ENDIF21
    :ELSE21
        REM === prepare PDF file ===
        %noweave% %sourceFile% %packageNameFile%
        PUSHD %tempDirectory%
        %pdfLaTeX% -interaction=batchmode ploy-ada.tex
        POPD
        MOVE >NUL /Y %tempDirectory%\ploy-ada.pdf doc
    :ENDIF21
:ENDIF2

REM ============================================
REM ========== MAKING PLOY EXECUTABLE ==========
REM ============================================

IF %processingMode% == document GOTO ENDIF3
    SET isOkay=T
    IF "%gnuAdaMake%" == ""   SET isOkay=F
    IF "%gtkAdaIncludeDirectory%" == ""  SET isOkay=F

    IF %isOkay% == T GOTO ELSE31
        SET message=%progName%: no GNU Ada compiler or GtkAda
        SET message=%message% - cannot produce executable
        ECHO %message%
	GOTO ENDIF31
    :ELSE31
        SET targetDirectory=%currentDirectory%\ada-source

        REM --- Ada compilation options ---
        SET optimizationLevel=3
        SET compilationOptions=-g -gnatf -gnat2012 -gnatwA -gnatW8
        SET compilationOptions=%compilationOptions% -fstack-check -E -i
        SET compilationOptions=%compilationOptions% -O%optimizationLevel%
        SET compilationOptions=%compilationOptions% -aI%gtkAdaIncludeDirectory%
        SET compilationOptions=%compilationOptions% -aL%gtkAdaIncludeDirectory%

        REM libraries are args, gtkada and the resource file with icons
        SET libraries=-shared -largs -lgtkada ployres.o

        REM --------------------------------------------
        REM -- generation of Ada files for base layer --
        REM --------------------------------------------
	%adaTangle% %sourceFile% base.ads %targetDirectory%
	%adaTangle% %sourceFile% base-assertion.ads %targetDirectory%
	%adaTangle% %sourceFile% base-assertion.adb %targetDirectory%
	%adaTangle% %sourceFile% base-boolean.ads %targetDirectory%
	%adaTangle% %sourceFile% base-boundedscalar.ads %targetDirectory%
	%adaTangle% %sourceFile% base-boundedscalar.adb %targetDirectory%
	%adaTangle% %sourceFile% base-character.ads %targetDirectory%
	%adaTangle% %sourceFile% base-conditionalexpression.ads %targetDirectory%
	%adaTangle% %sourceFile% base-error.ads %targetDirectory%
	%adaTangle% %sourceFile% base-error.adb %targetDirectory%
	%adaTangleP% %sourceFile% base-file .ads %targetDirectory%
	%adaTangleP% %sourceFile% base-file .adb %targetDirectory%
	%adaTangleP% %sourceFile% base-fingerprint .ads %targetDirectory%
	%adaTangleP% %sourceFile% base-fingerprint .adb %targetDirectory%
	%adaTangle% %sourceFile% base-genericconditionalexpression.ads %targetDirectory%
	%adaTangle% %sourceFile% base-genericconditionalexpression.adb %targetDirectory%
	%adaTangle% %sourceFile% base-generictrace.ads %targetDirectory%
	%adaTangle% %sourceFile% base-generictrace.adb %targetDirectory%
	%adaTangle% %sourceFile% base-homogenousonedmap.ads %targetDirectory%
	%adaTangle% %sourceFile% base-homogenousonedmap.adb %targetDirectory%
	%adaTangle% %sourceFile% base-integer.ads %targetDirectory%
	%adaTangle% %sourceFile% base-integer.adb %targetDirectory%
	%adaTangleP% %sourceFile% base-list .ads %targetDirectory%
	%adaTangleP% %sourceFile% base-list .adb %targetDirectory%
	%adaTangle% %sourceFile% base-longnatural.ads %targetDirectory%
	%adaTangle% %sourceFile% base-onedmap.ads %targetDirectory%
	%adaTangle% %sourceFile% base-onedmap.adb %targetDirectory%
	%adaTangle% %sourceFile% base-persistentstringmap.ads %targetDirectory%
	%adaTangle% %sourceFile% base-persistentstringmap.adb %targetDirectory%
	%adaTangle% %sourceFile% base-rsc.ads %targetDirectory%
	%adatangle% %sourceFile% base-rsc.adb %targetDirectory%
	%adaTangle% %sourceFile% base-set.ads %targetDirectory%
	%adaTangle% %sourceFile% base-set.adb %targetDirectory%
	%adaTangle% %sourceFile% base-settingsstorage.ads %targetDirectory%
	%adaTangle% %sourceFile% base-settingsstorage_windows.adb %targetDirectory%
	%adaTangle% %sourceFile% base-staticallocator.ads %targetDirectory%
	%adaTangle% %sourceFile% base-staticallocator.adb %targetDirectory%
	%adaTangle% %sourceFile% base-string.ads %targetDirectory%
	%adaTangle% %sourceFile% base-string.adb %targetDirectory%
	%adaTangle% %sourceFile% base-time.ads %targetDirectory%
	%adaTangle% %sourceFile% base-time.adb %targetDirectory%
	%adaTangle% %sourceFile% base-trace.ads %targetDirectory%
	%adaTangle% %sourceFile% base-trace.adb %targetDirectory%
	%adaTangle% %sourceFile% base-userinterface.ads %targetDirectory%
	%adaTangle% %sourceFile% base-userinterface.adb %targetDirectory%

        REM ----------------------------------------------
        REM -- generation of Ada files for engine layer --
        REM ----------------------------------------------
	%adaTangle% %sourceFile% engine.ads %targetDirectory%
	%adaTangle% %sourceFile% engine-board.ads %targetDirectory%
	%adaTangle% %sourceFile% engine-board.adb %targetDirectory%
	%adaTangleP% %sourceFile% engine-configuration .ads %targetDirectory%
	%adaTangleP% %sourceFile% engine-configuration .adb %targetDirectory%
	%adaTangleP% %sourceFile% engine-game .ads %targetDirectory%
	%adaTangleP% %sourceFile% engine-game .adb %targetDirectory%
	%adaTangle% %sourceFile% engine-gamevariant.ads %targetDirectory%
	%adaTangle% %sourceFile% engine-gamevariant.adb %targetDirectory%
	%adaTangleP% %sourceFile% engine-move .ads %targetDirectory%
	%adaTangleP% %sourceFile% engine-move .adb %targetDirectory%
	%adaTangleP% %sourceFile% engine-movelist .ads %targetDirectory%
	%adaTangleP% %sourceFile% engine-movelist .adb %targetDirectory%
	%adaTangleP% %sourceFile% engine-piece .ads %targetDirectory%
	%adaTangleP% %sourceFile% engine-piece .adb %targetDirectory%
	%adaTangleP% %sourceFile% engine-player .ads %targetDirectory%
	%adaTangleP% %sourceFile% engine-player .adb %targetDirectory%
        %adaTangleP% %sourceFile% engine-player -propertylist.ads %targetDirectory%
        %adaTangleP% %sourceFile% engine-player -propertylist.adb %targetDirectory%
	%adaTangleP% %sourceFile% engine-player -takeovermap.ads %targetDirectory%
	%adaTangle% %sourceFile% engine-timecontrol.ads %targetDirectory%
	%adaTangle% %sourceFile% engine-timecontrol.adb %targetDirectory%

        REM ----------------------------------------------------
        REM -- generation of Ada files for presentation layer --
        REM ----------------------------------------------------
	%adaTangle% %sourceFile% gui_toolkit.ads %targetDirectory%
	%adaTangle% %sourceFile% presentation.ads %targetDirectory%
	%adaTangleP% %sourceFile% presentation-configurationview .ads %targetDirectory%
	%adaTangleP% %sourceFile% presentation-configurationview .adb %targetDirectory%
	%adaTangle% %sourceFile% presentation-dialogsupport.ads %targetDirectory%
	%adaTangle% %sourceFile% presentation-dialogsupport.adb %targetDirectory%
	%adaTangle% %sourceFile% presentation-graphics.ads %targetDirectory%
	%adaTangle% %sourceFile% presentation-graphics.adb %targetDirectory%
	%adaTangleP% %sourceFile% presentation-movelistview .ads %targetDirectory%
	%adaTangleP% %sourceFile% presentation-movelistview .adb %targetDirectory%
	%adaTangle% %sourceFile% presentation-pieceview.ads %targetDirectory%
	%adaTangle% %sourceFile% presentation-pieceview.adb %targetDirectory%
	%adaTangle% %sourceFile% presentation-preferencesdialog.ads %targetDirectory%
	%adaTangle% %sourceFile% presentation-preferencesdialog.adb %targetDirectory%

        REM ------------------------------------------------
        REM -- generation of Ada files for toplevel layer --
        REM ------------------------------------------------
	%adaTangle% %sourceFile% toplevel.ads %targetDirectory%
	%adaTangle% %sourceFile% toplevel-action.ads %targetDirectory%
	%adaTangle% %sourceFile% toplevel-action.adb %targetDirectory%
	%adaTangle% %sourceFile% toplevel-dialogmanager.ads %targetDirectory%
	%adaTangle% %sourceFile% toplevel-dialogmanager.adb %targetDirectory%
	%adaTangle% %sourceFile% toplevel-mainwindow.ads %targetDirectory%
	%adaTangle% %sourceFile% toplevel-mainwindow.adb %targetDirectory%

        REM ---------------------------------------------
        REM -- generation of Ada file for main program --
        REM ---------------------------------------------
        %adaTangle% %sourceFile% ploy.adb %targetDirectory%

        REM -----------------------------------------
        REM -- renaming of platform-specific files --
        REM -----------------------------------------
        MOVE /Y %targetDirectory%\base-settingsstorage_windows.adb %targetDirectory%\base-settingsstorage.adb

        REM -----------------
        REM -- compilation --
        REM -----------------
        PUSHD %sourceDirectory%
        %gnuWindres% -i ploy.rc -o %targetDirectory%\ployres.o
        POPD
        PUSHD %targetDirectory%
        %gnuAdaMake% %compilationOptions% -mwindows ploy.adb %libraries%
        IF EXIST ploy.exe (  MOVE >NUL /Y ploy.exe %executableDirectory%  )
        POPD
    :ENDIF31
:ENDIF3
